@@ -6,7 +6,7 @@
  */
 
 #include "boost/date_time/gregorian/gregorian.hpp"
-#include "boost/date_time/testfrmwk.hpp"
+#include "../testfrmwk.hpp"
 
 int
 main() 
